// This file is generated by kconfig_compiler_kf6 from notificationsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef NOTIFICATIONMANAGER_NOTIFICATIONSETTINGS_H
#define NOTIFICATIONMANAGER_NOTIFICATIONSETTINGS_H

#include "settings.h"
#include "notificationmanager_export.h"

#include <qglobal.h>
#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace NotificationManager {

class NOTIFICATIONMANAGER_EXPORT NotificationSettings : public KConfigSkeleton
{
  Q_OBJECT
  public:

    NotificationSettings( QObject *parent = nullptr );
    ~NotificationSettings() override;

    /**
      Set CriticalInDndMode
    */
    void setCriticalInDndMode( bool v )
    {
      if (v != mCriticalInDndMode && !isCriticalInDndModeImmutable()) {
        mCriticalInDndMode = v;
        Q_EMIT CriticalInDndModeChanged();
      }
    }

    Q_PROPERTY(bool criticalInDndMode READ criticalInDndMode WRITE setCriticalInDndMode NOTIFY CriticalInDndModeChanged)
    Q_PROPERTY(bool isCriticalInDndModeImmutable READ isCriticalInDndModeImmutable CONSTANT)
    Q_PROPERTY(bool defaultCriticalInDndModeValue READ defaultCriticalInDndModeValue CONSTANT)
    /**
      Get CriticalInDndMode
    */
    bool criticalInDndMode() const
    {
      return mCriticalInDndMode;
    }

    /**
      Is CriticalInDndMode Immutable
    */
    bool isCriticalInDndModeImmutable() const
    {
      return isImmutable( QStringLiteral( "CriticalInDndMode" ) );
    }

    /**
      Get CriticalInDndMode default value
    */
    bool defaultCriticalInDndModeValue() const
    {
        return defaultCriticalInDndModeValue_helper();
    }

    /**
      Set LowPriorityPopups
    */
    void setLowPriorityPopups( bool v )
    {
      if (v != mLowPriorityPopups && !isLowPriorityPopupsImmutable()) {
        mLowPriorityPopups = v;
        Q_EMIT LowPriorityPopupsChanged();
      }
    }

    Q_PROPERTY(bool lowPriorityPopups READ lowPriorityPopups WRITE setLowPriorityPopups NOTIFY LowPriorityPopupsChanged)
    Q_PROPERTY(bool isLowPriorityPopupsImmutable READ isLowPriorityPopupsImmutable CONSTANT)
    Q_PROPERTY(bool defaultLowPriorityPopupsValue READ defaultLowPriorityPopupsValue CONSTANT)
    /**
      Get LowPriorityPopups
    */
    bool lowPriorityPopups() const
    {
      return mLowPriorityPopups;
    }

    /**
      Is LowPriorityPopups Immutable
    */
    bool isLowPriorityPopupsImmutable() const
    {
      return isImmutable( QStringLiteral( "LowPriorityPopups" ) );
    }

    /**
      Get LowPriorityPopups default value
    */
    bool defaultLowPriorityPopupsValue() const
    {
        return defaultLowPriorityPopupsValue_helper();
    }

    /**
      Set LowPriorityHistory
    */
    void setLowPriorityHistory( bool v )
    {
      if (v != mLowPriorityHistory && !isLowPriorityHistoryImmutable()) {
        mLowPriorityHistory = v;
        Q_EMIT LowPriorityHistoryChanged();
      }
    }

    Q_PROPERTY(bool lowPriorityHistory READ lowPriorityHistory WRITE setLowPriorityHistory NOTIFY LowPriorityHistoryChanged)
    Q_PROPERTY(bool isLowPriorityHistoryImmutable READ isLowPriorityHistoryImmutable CONSTANT)
    Q_PROPERTY(bool defaultLowPriorityHistoryValue READ defaultLowPriorityHistoryValue CONSTANT)
    /**
      Get LowPriorityHistory
    */
    bool lowPriorityHistory() const
    {
      return mLowPriorityHistory;
    }

    /**
      Is LowPriorityHistory Immutable
    */
    bool isLowPriorityHistoryImmutable() const
    {
      return isImmutable( QStringLiteral( "LowPriorityHistory" ) );
    }

    /**
      Get LowPriorityHistory default value
    */
    bool defaultLowPriorityHistoryValue() const
    {
        return defaultLowPriorityHistoryValue_helper();
    }

    /**
      Set PopupPosition
    */
    void setPopupPosition( int v )
    {
      if (v != mPopupPosition && !isPopupPositionImmutable()) {
        mPopupPosition = v;
        Q_EMIT PopupPositionChanged();
      }
    }

    Q_PROPERTY(int popupPosition READ popupPosition WRITE setPopupPosition NOTIFY PopupPositionChanged)
    Q_PROPERTY(bool isPopupPositionImmutable READ isPopupPositionImmutable CONSTANT)
    Q_PROPERTY(int defaultPopupPositionValue READ defaultPopupPositionValue CONSTANT)
    /**
      Get PopupPosition
    */
    int popupPosition() const
    {
      return mPopupPosition;
    }

    /**
      Is PopupPosition Immutable
    */
    bool isPopupPositionImmutable() const
    {
      return isImmutable( QStringLiteral( "PopupPosition" ) );
    }

    /**
      Get PopupPosition default value
    */
    int defaultPopupPositionValue() const
    {
        return defaultPopupPositionValue_helper();
    }

    /**
      Set PopupTimeout
    */
    void setPopupTimeout( int v )
    {
      if (v != mPopupTimeout && !isPopupTimeoutImmutable()) {
        mPopupTimeout = v;
        Q_EMIT PopupTimeoutChanged();
      }
    }

    Q_PROPERTY(int popupTimeout READ popupTimeout WRITE setPopupTimeout NOTIFY PopupTimeoutChanged)
    Q_PROPERTY(bool isPopupTimeoutImmutable READ isPopupTimeoutImmutable CONSTANT)
    Q_PROPERTY(int defaultPopupTimeoutValue READ defaultPopupTimeoutValue CONSTANT)
    /**
      Get PopupTimeout
    */
    int popupTimeout() const
    {
      return mPopupTimeout;
    }

    /**
      Is PopupTimeout Immutable
    */
    bool isPopupTimeoutImmutable() const
    {
      return isImmutable( QStringLiteral( "PopupTimeout" ) );
    }

    /**
      Get PopupTimeout default value
    */
    int defaultPopupTimeoutValue() const
    {
        return defaultPopupTimeoutValue_helper();
    }

    /**
      Set ShowPopupTimeout
    */
    void setShowPopupTimeout( bool v )
    {
      if (v != mShowPopupTimeout && !isShowPopupTimeoutImmutable()) {
        mShowPopupTimeout = v;
        Q_EMIT ShowPopupTimeoutChanged();
      }
    }

    Q_PROPERTY(bool showPopupTimeout READ showPopupTimeout WRITE setShowPopupTimeout NOTIFY ShowPopupTimeoutChanged)
    Q_PROPERTY(bool isShowPopupTimeoutImmutable READ isShowPopupTimeoutImmutable CONSTANT)
    Q_PROPERTY(bool defaultShowPopupTimeoutValue READ defaultShowPopupTimeoutValue CONSTANT)
    /**
      Get ShowPopupTimeout
    */
    bool showPopupTimeout() const
    {
      return mShowPopupTimeout;
    }

    /**
      Is ShowPopupTimeout Immutable
    */
    bool isShowPopupTimeoutImmutable() const
    {
      return isImmutable( QStringLiteral( "ShowPopupTimeout" ) );
    }

    /**
      Get ShowPopupTimeout default value
    */
    bool defaultShowPopupTimeoutValue() const
    {
        return defaultShowPopupTimeoutValue_helper();
    }


    enum {
      signalCriticalInDndModeChanged = 1,
      signalLowPriorityPopupsChanged = 2,
      signalLowPriorityHistoryChanged = 3,
      signalPopupPositionChanged = 4,
      signalPopupTimeoutChanged = 5,
      signalShowPopupTimeoutChanged = 6
    };

  Q_SIGNALS:
    void CriticalInDndModeChanged();

    void LowPriorityPopupsChanged();

    void LowPriorityHistoryChanged();

    void PopupPositionChanged();

    void PopupTimeoutChanged();

    void ShowPopupTimeoutChanged();

  private:
    void itemChanged(quint64 signalFlag);

  protected:

    // Notifications
    bool mCriticalInDndMode;
    bool defaultCriticalInDndModeValue_helper() const;
    bool mLowPriorityPopups;
    bool defaultLowPriorityPopupsValue_helper() const;
    bool mLowPriorityHistory;
    bool defaultLowPriorityHistoryValue_helper() const;
    int mPopupPosition;
    int defaultPopupPositionValue_helper() const;
    int mPopupTimeout;
    int defaultPopupTimeoutValue_helper() const;
    bool mShowPopupTimeout;
    bool defaultShowPopupTimeoutValue_helper() const;
};

}

#endif

