
#ifndef NOTIFICATIONMANAGER_EXPORT_H
#define NOTIFICATIONMANAGER_EXPORT_H

#ifdef NOTIFICATIONMANAGER_STATIC_DEFINE
#  define NOTIFICATIONMANAGER_EXPORT
#  define NOTIFICATIONMANAGER_NO_EXPORT
#else
#  ifndef NOTIFICATIONMANAGER_EXPORT
#    ifdef notificationmanager_EXPORTS
        /* We are building this library */
#      define NOTIFICATIONMANAGER_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define NOTIFICATIONMANAGER_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef NOTIFICATIONMANAGER_NO_EXPORT
#    define NOTIFICATIONMANAGER_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef NOTIFICATIONMANAGER_DEPRECATED
#  define NOTIFICATIONMANAGER_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef NOTIFICATIONMANAGER_DEPRECATED_EXPORT
#  define NOTIFICATIONMANAGER_DEPRECATED_EXPORT NOTIFICATIONMANAGER_EXPORT NOTIFICATIONMANAGER_DEPRECATED
#endif

#ifndef NOTIFICATIONMANAGER_DEPRECATED_NO_EXPORT
#  define NOTIFICATIONMANAGER_DEPRECATED_NO_EXPORT NOTIFICATIONMANAGER_NO_EXPORT NOTIFICATIONMANAGER_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef NOTIFICATIONMANAGER_NO_DEPRECATED
#    define NOTIFICATIONMANAGER_NO_DEPRECATED
#  endif
#endif

#endif /* NOTIFICATIONMANAGER_EXPORT_H */
